@@ -3,7 +3,7 @@
 /*
  *  T50 - Experimental Mixed Packet Injector
  *
- *  Copyright (C) 2010 - 2015 - T50 developers
+ *  Copyright (C) 2010 - 2019 - T50 developers
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include <t50_modules.h>
 #include <t50_randomizer.h>
 
-static uint32_t ospf_hdr_len ( const uint32_t, const int, const int, const _Bool );
+static size_t ospf_hdr_len ( uint32_t, int, int, int );
 static void ospf_lsupdate ( const config_options_T *const restrict, void **restrict, struct ospf_lsa_hdr *restrict );
 
 /**
@@ -40,12 +40,12 @@ static void ospf_lsupdate ( const config_options_T *const restrict, void **restr
  * @param co Pointer to T50 configuration structure.
  * @param size Pointer to packet size (updated by the function).
  */
-void ospf ( const config_options_T *const restrict co, uint32_t *restrict size )
+void ospf ( const config_options_T *const restrict co, size_t *restrict size )
 {
-  uint32_t length,
-           ospf_length, /* OSPF header length. */
-           counter,
-           stemp;
+  size_t length,
+         ospf_length, /* OSPF header length. */
+         counter,
+         stemp;
 
   uint8_t ospf_options, /* OSPF options? */
           lls;          /* OSPF LLS header? */
@@ -94,7 +94,7 @@ void ospf ( const config_options_T *const restrict co, uint32_t *restrict size )
                       ospf_tlv_len ( co->ospf.type, lls, co->ospf.auth ) );
 
   /* OSPF Header structure making a pointer to  IP Header structure. */
-  ospf          = ( struct ospf_hdr * ) ( ( unsigned char * ) ( ip + 1 ) + length );
+  ospf          = ( void * ) ( ip + 1 ) + length;
   ospf->version = OSPFVERSION;
   ospf->type    = co->ospf.type;
 
@@ -202,8 +202,10 @@ void ospf ( const config_options_T *const restrict co, uint32_t *restrict size )
 
       /* Dealing with neighbor address(es). */
       /* NOTE: Assume co->ospf.neighbor > 0. */
-      for ( counter = 0; likely ( counter < co->ospf.neighbor ); counter++ )
-        *buffer.inaddr_ptr++ = INADDR_RND ( co->ospf.address[counter] );
+      counter = 0;
+
+      while ( counter < co->ospf.neighbor )
+        *buffer.inaddr_ptr++ = INADDR_RND ( co->ospf.address[counter++] );
 
       break;
 
@@ -392,7 +394,9 @@ build_ospf_lsupdate:
   stemp = auth_hmac_md5_len ( co->ospf.auth );
 
   /* NOTE: Assume stemp > 0. */
-  for ( counter = 0; likely ( counter < stemp ); counter++ )
+  counter = 0;
+
+  while ( counter++ < stemp )
     *buffer.byte_ptr++ = RANDOM();
 
   /*
@@ -482,7 +486,9 @@ build_ospf_lsupdate:
         stemp = auth_hmac_md5_len ( co->ospf.auth );
 
         /* NOTE: Assume stemp > 0. */
-        for ( counter = 0; likely ( counter < stemp ); counter++ )
+        counter = 0;
+
+        while ( counter++ < stemp )
           *buffer.byte_ptr++ = RANDOM();
 
         /*
@@ -518,18 +524,18 @@ build_ospf_lsupdate:
     /* Computing the checksum. */
     ospf->check   = co->bogus_csum ?
                     RANDOM() :
-                    htons ( cksum ( ospf, ( uint32_t ) ( buffer.ptr - ( void * ) ospf ) ) );
+                    htons ( cksum ( ospf, ( size_t ) buffer.ptr - ( size_t ) ospf ) );
 
   gre_checksum ( packet, co, *size );
 }
 
 /* OSPF header size calculation. */
-uint32_t ospf_hdr_len ( const uint32_t type,
-                        const int neighbor,
-                        const int lsa_type,
-                        const _Bool dd_include_lsa )
+size_t ospf_hdr_len ( uint32_t type,
+                      int neighbor,
+                      int lsa_type,
+                      int dd_include_lsa )
 {
-  uint32_t size;
+  size_t size;
 
   /*
    * The code starts with size '0' and it accumulates all the required
