@@ -3,7 +3,7 @@
 /*
  *  T50 - Experimental Mixed Packet Injector
  *
- *  Copyright (C) 2010 - 2015 - T50 developers
+ *  Copyright (C) 2010 - 2019 - T50 developers
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include <t50_modules.h>
 #include <t50_randomizer.h>
 
-static  uint32_t rsvp_objects_len ( const uint8_t, const uint8_t, const uint8_t, const uint8_t );
+static size_t rsvp_objects_len ( const uint8_t, const uint8_t, const uint8_t, const uint8_t );
 
 /**
  * RSVP packet header configuration.
@@ -39,11 +39,10 @@ static  uint32_t rsvp_objects_len ( const uint8_t, const uint8_t, const uint8_t,
  * @param co Pointer to T50 configuration structure.
  * @param size Pointer to packet size (updated by the function).
  */
-void rsvp ( const config_options_T *const restrict co, uint32_t *restrict size )
+void rsvp ( const config_options_T *const restrict co, size_t *restrict size )
 {
-  uint32_t greoptlen,       /* GRE options size. */
-           objects_length,  /* RSVP objects length. */
-           counter;
+  size_t greoptlen,       /* GRE options size. */
+         objects_length;  /* RSVP objects length. */
 
   /* Packet and Checksum. */
   memptr_T buffer;
@@ -76,7 +75,7 @@ void rsvp ( const config_options_T *const restrict co, uint32_t *restrict size )
                       objects_length );
 
   /* RSVP Header structure making a pointer to IP Header structure. */
-  rsvp           = ( struct rsvp_common_hdr * ) ( ( unsigned char * ) ( ip + 1 ) + greoptlen );
+  rsvp           = ( void * ) ( ip + 1 ) + greoptlen;
   rsvp->flags    = __RND ( co->rsvp.flags );
   rsvp->version  = RSVPVERSION;
   rsvp->type     = co->rsvp.type;
@@ -534,6 +533,8 @@ void rsvp ( const config_options_T *const restrict co, uint32_t *restrict size )
          co->rsvp.type == RSVP_MESSAGE_TYPE_RESVTEAR ||
          co->rsvp.type == RSVP_MESSAGE_TYPE_RESVERR )
     {
+      uint32_t counter;
+
       /*
        * Resource ReSerVation Protocol (RSVP) (RFC 2205)
        *
@@ -557,8 +558,10 @@ void rsvp ( const config_options_T *const restrict co, uint32_t *restrict size )
 
       /* Dealing with scope address(es). */
       /* NOTE: Assume co->rsvp.scope > 0. */
-      for ( counter = 0; likely ( counter < co->rsvp.scope ) ; counter++ )
-        *buffer.inaddr_ptr++ = INADDR_RND ( co->rsvp.address[counter] );
+      counter = 0;
+
+      while ( counter < co->rsvp.scope )
+        *buffer.inaddr_ptr++ = INADDR_RND ( co->rsvp.address[counter++] );
     }
 
     /*
@@ -593,16 +596,16 @@ void rsvp ( const config_options_T *const restrict co, uint32_t *restrict size )
   /* Computing the checksum. */
   rsvp->check   = co->bogus_csum ?
                   RANDOM() :
-                  htons ( cksum ( rsvp, buffer.ptr - ( void * ) rsvp ) );
+                  htons ( cksum ( rsvp, ( size_t ) buffer.ptr - ( size_t ) rsvp ) );
 
   /* GRE Encapsulation takes place. */
   gre_checksum ( packet, co, *size );
 }
 
 /* RSVP objects size claculation. */
-uint32_t rsvp_objects_len ( const uint8_t type, const uint8_t scope, const uint8_t adspec, const uint8_t tspec )
+size_t rsvp_objects_len ( const uint8_t type, const uint8_t scope, const uint8_t adspec, const uint8_t tspec )
 {
-  uint32_t size;
+  size_t size;
 
   /*
    * The code starts with the size of SESSION Object Class  (according
