@@ -3,7 +3,7 @@
 /*
  *  T50 - Experimental Mixed Packet Injector
  *
- *  Copyright (C) 2010 - 2015 - T50 developers
+ *  Copyright (C) 2010 - 2019 - T50 developers
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,20 +23,15 @@
 #include <t50_shuffle.h>
 #include <t50_randomizer.h>
 
-// NOTE: It is safe to use uint32_t instead of uint32_t 'cause
-//       the index buffer will never be greater than the number
-//       of available protocols.
-void shuffle ( uint32_t *p, uint32_t size )
+// Knuth-Fisher-Yates (this one) is faster!
+void shuffle ( uint32_t *p, size_t size )
 {
-  uint32_t i, j;
+  size_t i;
 
-  for ( i = 0; i < size - 2; i++ )
+  while ( size )
   {
-    // NOTE: This routine will be called once each 'size'
-    //       main loop iterations. This division will not
-    //       slow things down very much...
-    j = ( RANDOM() % ( size - i ) ) + i;
-    swap ( p[i], p[j] );
+    i = RANDOM() % size--;
+    swap ( p[size], p[i] );
   }
 }
 
